import { Buff } from '@cmdcode/buff';
import { consume_keypairs, get_keypair } from './keypair.js';
import CONST from '../const.js';
export function decode_psbt(psbthex) {
    const stream = Buff.hex(psbthex).stream;
    const magic = stream.read(5).hex;
    if (magic !== CONST.PSBT_MAGIC_BYTES) {
        throw new Error('invalid magic for psbt: ' + magic);
    }
    const global = consume_keypairs(stream, CONST.PSBT_GLOBAL_TYPE_MAP);
    const vin_count = get_input_count(global);
    const out_count = get_output_count(global);
    const inputs = [];
    for (let i = 0; i < vin_count; i++) {
        const pairs = consume_keypairs(stream, CONST.PSBT_INPUT_TYPE_MAP);
        inputs.push(pairs);
    }
    const outputs = [];
    for (let i = 0; i < out_count; i++) {
        const pairs = consume_keypairs(stream, CONST.PSBT_OUTPUT_TYPE_MAP);
        outputs.push(pairs);
    }
    if (stream.size !== 0) {
        throw new Error('stream has excess bytes: ' + new Buff(stream.data).hex);
    }
    return { global, inputs, outputs };
}
function get_input_count(keys) {
    const kp = get_keypair(keys, 'INPUT_COUNT');
    return Buff.bytes(kp.value).num;
}
function get_output_count(keys) {
    const kp = get_keypair(keys, 'OUTPUT_COUNT');
    return Buff.bytes(kp.value).num;
}
