import { Buff } from '@cmdcode/buff';
import { assert } from '../util/index.js';
export function consume_keypairs(stream, keymap) {
    const pairs = [];
    while (stream.peek(1).num !== 0x00) {
        const kp = decode_keypair(stream, keymap);
        pairs.push(kp);
    }
    stream.read(1);
    return pairs;
}
export function decode_keypair(stream, keymap) {
    const klen = stream.read_varint();
    const kdat = stream.read(klen).stream;
    const type = kdat.read_varint();
    let key = (kdat.size !== 0)
        ? kdat.read(kdat.size).hex
        : null;
    let label = keymap[type];
    if (type === 0xFC) {
        assert.exists(key);
        const pdat = Buff.bytes(key).stream;
        const plen = pdat.read_varint();
        label = pdat.read(plen).str;
        key = pdat.read(pdat.size).hex;
    }
    const vlen = stream.read_varint();
    const value = stream.read(vlen).hex;
    return { key, label, type, value };
}
export function encode_keypair(kp) {
    const { key, label, type, value } = kp;
    let kb = new Buff(0, 0);
    if (type === 0xFC) {
        assert.exists(key);
        const label_bytes = Buff.str(label);
        const label_vint = Buff.calc_varint(label_bytes.length);
        kb = Buff.join([label_vint, label_bytes, key]);
    }
    else if (key !== null) {
        kb = Buff.hex(key);
    }
    const type_bytes = Buff.num(type);
    const key_bytes = Buff.join([type_bytes, kb]);
    const key_vint = Buff.calc_varint(key_bytes.length);
    const value_bytes = Buff.hex(value);
    const value_vint = Buff.calc_varint(value_bytes.length);
    return Buff.join([key_vint, key_bytes, value_vint, value_bytes]).hex;
}
export function get_keypair(keys, label) {
    const kp = keys.find(e => e.label === label);
    assert.exists(kp, 'keypair not found: ' + label);
    return kp;
}
